# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import *
import random
import itertools

SCORES = [-9, -7, -6, -6, -5, -5, -4, -4, -4, -3, -3, -3, -2, -2, -2, 
  -2, -1, -1, -1, -1, -1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 
  6, 6, 6, 7, 7, 7, 8, 8, 9, 9, 10, 15, 
  1, 12]
MAX_ABS_SCORE = max(abs(s) for s in SCORES)
  
VISITED = -999
  
class ThinkAheadGameState(GameState):
    def reset(self):
        GameState.reset(self)
        self.extra_state = {'player_scores': [0,0], 'move_dim_index': 0}
        # fill grid with random values
        # make enough copies of SCORES value
        ncopies = self.grid.size // len(SCORES)
        if self.grid.size % len(SCORES) > 0 : ncopies += 1
        s = (SCORES*ncopies)[:self.grid.size]
        random.shuffle(s)
        for c,val in itertools.izip(self.itercoords(), s):
            self.grid[c] = val
        # start at "middle"
        self.extra_state['last_move'] = tuple(d//2 for d in self.dimensions())
        self.grid[self.extra_state['last_move']] = VISITED
            
    def player_score(self, pnum):
        return self.extra_state['player_scores'][pnum-1]
        
    def copy_extra_state(self):
        # hack: only return deep-copied player_scores, other attributes are set in successors()
        return {'player_scores': self.extra_state['player_scores'][:]}
        
    def successors(self):
        dim = self.dimensions()
        maxd = dim[self.extra_state['move_dim_index']]
        # check positions from last move varying the value of the "current" dimension
        c = list(self.extra_state['last_move'])
        for i in xrange(maxd):
            c[self.extra_state['move_dim_index']] = i
            pos = tuple(c)
            val = self.grid[pos]
            if val!=VISITED:
                nextgrid = self.grid.copy()
                nextgrid[pos] = VISITED
                nextstate = self.next_state_with_grid(nextgrid)
                # copy scores and move dimension
                nextstate.extra_state['player_scores'][self.current_player-1] += val
                nextstate.extra_state['move_dim_index'] = (1+self.extra_state['move_dim_index']) % len(dim)
                nextstate.extra_state['last_move'] = pos
                yield ( (pos,), nextstate)
                
    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        # move "active" cell and shrink destination
        moveanim = PGPieceAnimation.move_animation(tuple(self.extra_state['last_move']), move[0])
        delanim  = PGPieceAnimation.remove_animation(move[0])
        return PGPieceAnimation.AnimationGroup([moveanim, delanim])
                
